--
-- CustomWheels
--
-- written by Spartan086
--

CustomWheels = {};

CustomWheels.modDir = g_currentModDirectory;

function CustomWheels.prerequisitesPresent(specializations)
    return true; 
end;

function CustomWheels:load(xmlFile)
	--NOTE model.i3d aspect ratio is the most vital to match!
	local i = 1;
	while true do
		local baseString = string.format("vehicle.CustomWheels.wheel(%d)", i-1);
		local wheelNode = Utils.indexToObject( self.components, getXMLString(xmlFile, baseString.."#transformNode") );
		if wheelNode == nil then
			--print("FOUND");
			--print("FOUND "..(i-1).." wheels");
			--print("FOUND");
			break;
		end;
		
		local defaultWidth = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#defaultWidth"), 500);
		local newWidth = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#newWidth"), 500);
		local defaultRadial = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#defaultRadial"), 30);
		local newRadial = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#newRadial"), 30);
		local defaultAspectRatio = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#defaultAspectRatio"), 70);
		local newAspectRatio = Utils.getNoNil(getXMLFloat(xmlFile, baseString.."#newAspectRatio"), 70);
		
		local widthScale = newWidth/defaultWidth;
		local oldDiameter = (defaultWidth * defaultAspectRatio/100) * 2 + defaultRadial * 25.4;
		local newDiameter = (newWidth * newAspectRatio/100) * 2 + newRadial * 25.4;
		local heightScale = newDiameter/oldDiameter;
		
		if self.wheels[i] then
			self.wheels[i].width = newWidth/1000;
			self.wheels[i].radius = (newDiameter/2)/1000;
			self:updateWheelBase(self.wheels[i]);
		else
			print("CustomWheel error, wheel "..i.." does not exist!");
		end;
		setScale(wheelNode, widthScale, heightScale, heightScale);
		--print("Wheel requested: "..newWidth.."/"..newAspectRatio.."R"..newRadial);
		--print("Wheel set to: "..self.wheels[i].width.." radius "..self.wheels[i].radius);
		
		i = i + 1;
	end;




--[[


	--self.triggerCallback = SpecializationUtil.callSpecializationsFunction("triggerCallback"); 
	--self.setScale = SpecializationUtil.callSpecializationsFunction("setScale");
	--self.cusTrac = {};
	--self.cusTrac.wheels = {};
	--self.cusTrac.front = {};
	--self.cusTrac.rear = {};
	--self.cusTrac.inTrigger = false
	
	local i = 1;
	while true do
		if self.wheels[i] == nil then
			print("FOUND");
			print("FOUND "..i.." wheels");
			print("FOUND");
			break;
		end;
		self.cusTrac.wheels[i] = {};
		
		--TODO Read wheel.driveNode/wheel.repr instead of dynamicallyLoaded.linkNode!
		--self.cusTrac.wheels[i].transform = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.wheels.wheel(%d)#repr", i)) );
		self.cusTrac.wheels[i].transform = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.dynamicallyLoadedParts.dynamicallyLoadedPart(%d)#linkNode", i-1)) );
		i = i + 1;
	end;
	
	--Get and Set default scales
	self.cusTrac.defaultWidthScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.CustomWheels#defaultWidthScale"), 1.0);
	self.cusTrac.defaultRadiusScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.CustomWheels#defaultRadiusScale"), 1.0);
	
	
	self.cusTrac.defaultWidthScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.CustomWheels#defaultWidthScale"), 1.0);
	self.cusTrac.defaultRadiusScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.CustomWheels#defaultRadiusScale"), 1.0);
	
	
	self.cusTrac.tyreScaleW = self.cusTrac.defaultWidthScale;
	self.cusTrac.tyreScaleR = self.cusTrac.defaultRadiusScale;
	self:setScale();
	
	self.cusTrac.trigger = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.CustomWheels#trigger")) );
	print("triggerID "..self.cusTrac.trigger);
	
	--if self.cusTrac.trigger == nil then
		--print("[Error::CustomWheels] node is OK, but trigger could not be located. Check line "..tostring(i+1).." in your vehicle.xml");
	--end;
	
	--addTrigger( self.cusTrac.trigger, "triggerCallback", self );	
	--]]
end;

function CustomWheels:delete()
	--if self.cusTrac.trigger ~= nil then
		--removeTrigger(self.cusTrac.trigger);
	--end;
end;

function CustomWheels:readStream(streamId, connection)
end;

function CustomWheels:writeStream(streamId, connection)
end;

function CustomWheels:mouseEvent(posX, posY, isDown, isUp, button)
end;

function CustomWheels:keyEvent(unicode, sym, modifier, isDown)
end;

function CustomWheels:update(dt)
	--[[
	local delta = 0.1 * dt;
	if self.cusTrac.inTrigger then
		g_currentMission:addHelpButtonText( g_i18n:getText("SET_RADIUS_SCALEUP"), InputBinding.SET_RADIUS_SCALEUP );	
		g_currentMission:addHelpButtonText( g_i18n:getText("SET_RADIUS_SCALEDOWN"), InputBinding.SET_RADIUS_SCALEDOWN );	
		g_currentMission:addHelpButtonText( g_i18n:getText("SET_WIDTH_SCALEUP"), InputBinding.SET_WIDTH_SCALEUP );	
		g_currentMission:addHelpButtonText( g_i18n:getText("SET_WIDTH_SCALEDOWN"), InputBinding.SET_WIDTH_SCALEDOWN );	
		
		if InputBinding.isPressed(InputBinding.SET_RADIUS_SCALEUP)then
			self.cusTrac.tyreScaleR = self.cusTrac.tyreScaleR + delta/1000 ;
			
			self:setScale();
		end
		
		if InputBinding.isPressed(InputBinding.SET_RADIUS_SCALEDOWN)then
			self.cusTrac.tyreScaleR = self.cusTrac.tyreScaleR - delta/1000 ;
			
			self:setScale();
		end
		
		if InputBinding.isPressed(InputBinding.SET_WIDTH_SCALEUP)then
			self.cusTrac.tyreScaleW = self.cusTrac.tyreScaleW + delta/1000 ;
			
			self:setScale();
		end
		
		if InputBinding.isPressed(InputBinding.SET_WIDTH_SCALEDOWN)then
			self.cusTrac.tyreScaleW = self.cusTrac.tyreScaleW - delta/1000 ;
			
			self:setScale();
		end
		
	end;
	--]]
end;

function CustomWheels:onLeave()
end;

function CustomWheels:draw()
end;

function CustomWheels:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	--[[
	if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.cusTrac.inTrigger = true;
		print("trigger enter");
	elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.cusTrac.inTrigger = false;
		print("trigger leave");
	end;
	--]]
end;

function CustomWheels:setScale()
	--[[
		--TODO scale radius!
		--self.wheels[1].spring = 35;
		--self.wheels[1].radius = 0.55;
		--then call
		--self:updateWheelBase(self.wheels[1]);
		-- and maybe
		--self:updateWheelTireFriction(self.wheels[1])
	local i = 1;
	while true do
		if self.cusTrac.wheels[i] == nil then
			print("stopped scaling at wheel "..i);
			break;
		else
			print("scaling wheel "..i);
			if self.wheels[i] ~= nil then
				print("wheel OK");
			else
				print("self.wheels empty!");
			end;
			if self.cusTrac.wheels[i] ~= nil then
				print("cusTrac.wheel OK");
			else
				print("self.cusTrac.wheels empty!");
			end;
			self.wheels[i].width = self.cusTrac.wheels[i].defaultWidth * self.cusTrac.tyreScaleW;
			self.wheels[i].radius = self.cusTrac.wheels[i].defaultRadius * self.cusTrac.tyreScaleR;
			self:updateWheelBase(self.wheels[i]);
			setScale(self.cusTrac.wheels[i].transform, self.cusTrac.tyreScaleW, self.cusTrac.tyreScaleR, self.cusTrac.tyreScaleR);
		end;
		i = i + 1;
	end;
	--]]
end;