--
-- Tuning
-- @author  Spartan086
--


Tuning = {};

function Tuning.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function Tuning:load(xmlFile)
	--local torqueBoost = 1.23;
	local xmlString = "vehicle.gearboxMogli";
	self.tuning = {};
	self.tuning.isUpdated = true;
	self.tuning.used = false;
	self.tuning.orgTorqueCurve = {};
	self.tuning.newTorqueCurve = {};
	self.tuning.enabled = nil;
	self.tuning.enabledNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tuning#enabledNode"));
	local torqueBoost = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tuning#factor"), 1.2);
	self.tuning.addCost = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tuning#addCost"), 200);
	self.tuning.removeCost = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tuning#removeCost"), 50);
	self.tuning.upkeepCost = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tuning#upkeepCost"), 100);
	
	print(torqueBoost);
	

	--if self.mrGbMS ~= nil then
		local torqueI = 0;
		local torqueF = nil
		while true do
			local key = string.format(xmlString..".realEngine.torque(%d)", torqueI);
			local rpm = getXMLFloat(xmlFile, key.."#rpm");
			local torque = getXMLFloat(xmlFile, key.."#motorTorque");
			if torque == nil then
				torque = getXMLFloat(xmlFile, key.."#ptoTorque");
				if torque ~= nil then
					torque = torque / 0.94 --self.mrGbMS.TransmissionEfficiency
				end
			end		
			--local fuelUsageRatio = getXMLFloat(xmlFile, key.."#fuelUsageRatio");
			if torque == nil or rpm == nil then --or fuelUsageRatio==nil then
				print ("WARNING torque or rpm is nil");
				break;
			end;
			
			torqueF = Utils.getNoNil(getXMLFloat(xmlFile, xmlString..".realEngine#torqueFactor"), 1.1182033096926713947990543735225) / 1000
			torque = torque * torqueF 
	  
			self.tuning.orgTorqueCurve[torqueI+1] = {v=torque, time = rpm};
			self.tuning.newTorqueCurve[torqueI+1] = {v=torque*torqueBoost, time = rpm};
			
			torqueI = torqueI + 1;		
		end;
	--else
		--print ("WARNING self.mrGbMS is nil");
	--end;
		
	Tuning.loadDefMotors(self, xmlFile, torqueBoost);
	
	--[[
	print("---ORG CURVE---");
	for key,value in pairs(self.tuning.orgTorqueCurve) do
		print("found member " .. key);
	end
	print("---NEW CURVE---");
	for key,value in pairs(self.tuning.newTorqueCurve) do
		print("found member " .. key);
	end
	print("---OLD ENG CURVE---");
	for key,value in pairs(self.tuning.defMotor) do
		print("found member " .. key);
	end
	print("---NEW ENG CURVE---");
	for key,value in pairs(self.tuning.defTunedMotor) do
		print("found member " .. key);
	end
	]]--
end;

function Tuning.loadDefMotors(self, xmlFile, torqueBoost)

	local motorMinRpm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#minRpm"), 1000);
	local motorMaxRpm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#maxRpm"), 1800);
	local maxForwardSpeed = getXMLFloat(xmlFile, "vehicle.motor#maxForwardSpeed");
	local maxBackwardSpeed = getXMLFloat(xmlFile, "vehicle.motor#maxBackwardSpeed");
	if maxForwardSpeed ~= nil then
		maxForwardSpeed = maxForwardSpeed/3.6;
	end
	if maxBackwardSpeed ~= nil then
		maxBackwardSpeed = maxBackwardSpeed/3.6;
	end
	local brakeForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#brakeForce"), 10)*2;
	local lowBrakeForceScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#lowBrakeForceScale"), 0.5);
	local lowBrakeForceSpeedLimit = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#lowBrakeForceSpeedLimit"), 20)/3600;
	local forwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#forwardGearRatio"), 2);
	local backwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#backwardGearRatio"), 1.5);
	local maxForwardGearRatio = getXMLFloat(xmlFile, "vehicle.motor#maxForwardGearRatio");
	local minForwardGearRatio = getXMLFloat(xmlFile, "vehicle.motor#minForwardGearRatio");
	local maxBackwardGearRatio = getXMLFloat(xmlFile, "vehicle.motor#maxBackwardGearRatio");
	local minBackwardGearRatio = getXMLFloat(xmlFile, "vehicle.motor#minBackwardGearRatio");
	local rpmFadeOutRange = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#rpmFadeOutRange"), 20);
	local torqueScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#torqueScale"), 1);
	local ptoMotorRpmRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#ptoMotorRpmRatio"), 4);
	local maxTorque = 0;
	local torqueCurve = AnimCurve:new(linearInterpolator1);
	local torqueCurveTuned = AnimCurve:new(linearInterpolator1);
	local torqueI = 0;
	while true do
		local key = string.format("vehicle.motor.torque(%d)", torqueI);
		local normRpm = getXMLFloat(xmlFile, key.."#normRpm");
		local rpm;
		if normRpm == nil then
			rpm = getXMLFloat(xmlFile, key.."#rpm");
		else
			rpm = normRpm * motorMaxRpm;
		end
		local torque = getXMLFloat(xmlFile, key.."#torque");
		if torque == nil or rpm == nil then
			break;
		end;
		torqueCurve:addKeyframe({v=torque*torqueScale, time = rpm});
		torqueCurveTuned:addKeyframe({v=torque*torqueScale*torqueBoost, time = rpm});
		torqueI = torqueI +1;
		if torque*torqueScale > maxTorque then
			maxTorque = torque*torqueScale;
		end;
	end;
	self.tuning.defMotor = VehicleMotor:new(self, motorMinRpm, motorMaxRpm, maxForwardSpeed, maxBackwardSpeed, torqueCurve, brakeForce, forwardGearRatio, backwardGearRatio, minForwardGearRatio, maxForwardGearRatio, minBackwardGearRatio, maxBackwardGearRatio, ptoMotorRpmRatio, rpmFadeOutRange, maxTorque);
	self.tuning.defMotor:setLowBrakeForce(lowBrakeForceScale, lowBrakeForceSpeedLimit);
	self.tuning.defTunedMotor = VehicleMotor:new(self, motorMinRpm, motorMaxRpm, maxForwardSpeed, maxBackwardSpeed, torqueCurveTuned, brakeForce, forwardGearRatio, backwardGearRatio, minForwardGearRatio, maxForwardGearRatio, minBackwardGearRatio, maxBackwardGearRatio, ptoMotorRpmRatio, rpmFadeOutRange, maxTorque);
	self.tuning.defTunedMotor:setLowBrakeForce(lowBrakeForceScale, lowBrakeForceSpeedLimit);

end;

function Tuning:delete()
end;

function Tuning:readStream(streamId, connection)
end;

function Tuning:writeStream(streamId, connection)
end;

function Tuning:readUpdateStream(streamId, timestamp, connection)
end;

function Tuning:writeUpdateStream(streamId, connection, dirtyMask)
end;

function Tuning:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Tuning:keyEvent(unicode, sym, modifier, isDown)
end;

function Tuning:update(dt)
	if self.mrGbMS ~= nil and not self.mbClientInitDone20 then
		--print("Gearbox not ready yet");
		return;
	end;
	if self.tuning.enabled == nil then
		self.tuning.enabled = false; --getVisibility(self.tuning.enabledNode);
		setVisibility(self.tuning.enabledNode, false);
		return;
		--if self.tuning.enabled then
			--print("ONCE! ON");
		--else
			--print("ONCE! OFF");
		--end;
	end;
	--if self.motor == nil then 
		--print("No motor!?");
		--return;
	--end;
	
	if self.tuning.enabledNode ~= nil then
		self.tuning.toggle = getVisibility(self.tuning.enabledNode);
		
		
		-- Only change if either value has changed! (NOTE: universalToggleParts changes the self.tuning.toggle value by setting visibility to true or false.
		if self.tuning.toggle and not self.tuning.enabled then
			-- Enable
			-- Set to tuned motor
			self.motor = self.tuning.defTunedMotor;
			if self.mrGbMS ~= nil then
				self.mrGbMS.Engine.torqueValues = self.tuning.newTorqueCurve;
				self.mrGbML.motor = mrGearboxMogliMotor:new( self, self.motor );		
				self.mrGbMB.motor = self.tuning.defTunedMotor;
			end;
			self.tuning.enabled = true;
		elseif not self.tuning.toggle and self.tuning.enabled then
			-- Disable
			-- Set to default motor
			self.motor = self.tuning.defMotor;
			if self.mrGbMS ~= nil then
				self.mrGbMS.Engine.torqueValues = self.tuning.orgTorqueCurve;
				self.mrGbML.motor = mrGearboxMogliMotor:new( self, self.motor );		
				self.mrGbMB.motor = self.tuning.defMotor;	
			end;
			self.tuning.enabled = false;
		end;
	end;
	
	if self:getIsActiveForInput() and self.isEntered then
		if self.tuning.enabled then
			g_currentMission:addHelpButtonText(g_i18n:getText("TXTUNINGOFF"), InputBinding.TXTUNING);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("TXTUNINGON"), InputBinding.TXTUNING);
		end;
		--[[
		if self.tuning.enabled then
			g_currentMission:addHelpButtonText(g_i18n:getText("TXTUNINGOFF").." "..self.tuning.removeCost..g_i18n:getText("currency"), InputBinding.TXTUNING);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("TXTUNINGON").." "..self.tuning.addCost..g_i18n:getText("currency"), InputBinding.TXTUNING);
		end;
		]]--
	end;
	
	
	-- Keep track of usage and add maintenance costs.
	self.tuning.used = self.tuning.used or self.tuning.enabled;
	if self.driveControl ~= nil then
		self.tuning.turnedOn = self.tuning.turnedOn or self.driveControl.manMotorStart.isMotorStarted;
	else
		self.tuning.turnedOn = self.tuning.turnedOn or self:getIsActive();
	end;
	
	
	if g_currentMission.environment.currentHour == 22 and self.tuning.isUpdated ~= false then
		self.tuning.isUpdated = false;
	end;
	
	if g_currentMission.environment.currentHour == 23 and self.tuning.isUpdated == false then
		if self.tuning.used and self.tuning.turnedOn then
			print("both on");
		elseif self.tuning.used then
			print("tuning on, vehicle off");
		elseif self.tuning.turnedOn then
			print("tuning off, vehicle on");
		else
			print("tuning off, vehicle off");
		end;
		if g_server ~= nil and self.tuning.used and self.tuning.turnedOn then 
			g_currentMission:addSharedMoney(-self.tuning.upkeepCost, "vehicleRunningCost");
		end;
		self.tuning.used = false;
		self.tuning.turnedOn = false;
		self.tuning.isUpdated = true;
	end;
	
end;

function Tuning:updateTick(dt)

end;

function Tuning:draw()
end;

function Tuning:onLeave()

end;

function Tuning:onEnter()

end;
